"""
Collection of B -> D** l nu BF calibration fitters.

Felix Metzner 2023
"""

from typing import Tuple, Dict, Type

from rdstar.latex_labels.latex_units import LatexUnits
from rdstar.comparison_tools.select_utilities import SimpleCutInfo
from rdstar.latex_labels.latex_observable_names import LatexObsNames
from rdstar.offline_analysis.observables.common_observables import Cols

from rdstar.offline_analysis.fitting.dedicated_fit_approach.dss_cal_fitters import fitter_dss_cal_first_approach
from rdstar.offline_analysis.fitting.dedicated_fit_approach.dss_cal_fitters import fitter_dss_cal_second_approach
from rdstar.offline_analysis.fitting.dedicated_fit_approach.dss_cal_fitters import fitter_dss_cal_third_approach

from rdstar.offline_analysis.fitting.dedicated_fit_approach.dss_cal_fitters.dss_cal_fitter_base import DssBFCorrFitterBase


__all__ = [
    "d_w_wo_star_pi_minus_d_w_wo_star_mass_selection_for_dss_cal",
    "dss_c_fitter_dict",
    "keys_to_evaluate",
]


d_w_wo_star_pi_minus_d_w_wo_star_mass_selection_for_dss_cal = SimpleCutInfo(
    cut_str=f"{Cols.diff_dstar_d_mass} > 0.3",
    cut_latex_label=rf"{LatexObsNames.d_st_st_delta_m[:-1]} > 0.3\,{LatexUnits.gev[1:]}",
    cut_var_col_name=Cols.diff_dstar_d_mass,
)  # type: SimpleCutInfo


dss_c_fitter_dict: Dict[str, Type[DssBFCorrFitterBase]] = {
    # First approach for Dss BF fits.
    "dss_c_first_approach": fitter_dss_cal_first_approach.DssBFCorrFitter,
    "dss_c_second_approach": fitter_dss_cal_second_approach.DssBFCorrFitter,
    "dss_c_third_approach": fitter_dss_cal_third_approach.DssBFCorrFitter,
}


keys_to_evaluate: Tuple[str, ...] = (
    # "dss_c_first_approach",
    # "dss_c_second_approach",
    "dss_c_third_approach",
)


assert all(k in dss_c_fitter_dict for k in keys_to_evaluate), [k for k in keys_to_evaluate if k not in dss_c_fitter_dict]
