"""
Defining Rest Of Event (ROE) Masks which are used to clean the set of
tracks and ECL clusters which are considered for the ROE.
Felix Metzner 2018
"""

from typing import Dict, List, Callable
from analysistools.processing import roe_masks_utils


def emptyROEMask(use_b2bii: bool) -> Dict[str, str]:
    return {
        "name": "EmptyROE",
        "track_cuts": "",
        "cluster_cuts": "",
    }


def cleanROEMask(use_b2bii: bool) -> Dict[str, str]:
    return {
        "name": "CleanROE",
        "track_cuts": "dr < 2 and abs(dz) < 4 and trackCurlerCloneIndicator == 0"
        if use_b2bii
        else "dr < 2 and abs(dz) < 4 and nCDCHits > 0",
        "cluster_cuts": "goodBelleGamma == 1" if use_b2bii else "goodGamma == 1",
    }


def michaelROEMask(use_b2bii: bool) -> Dict[str, str]:
    return {
        "name": "MichaelROE",
        "track_cuts": "dr < 2 and abs(dz) < 4" if use_b2bii else "dr < 2 and abs(dz) < 4 and nCDCHits > 0",
        "cluster_cuts": "goodBelleGamma == 1 and clusterBelleQuality == 0 and trackMatchType == 0"
        if use_b2bii
        else "goodGamma == 1 and trackMatchType == 0",
    }


def thomas_cluster_cuts(use_b2bii: bool) -> List[str]:
    cluster_cuts = [
        "goodBelleGamma == 1" if use_b2bii else "goodGamma == 1",
        "trackMatchType == 0",
        "abs(clusterTiming) < 58.6",
        "minC2TDist >= 26.8",
        "clusterE9E25 < 1.25",
    ]
    if use_b2bii:
        cluster_cuts.append("clusterBelleQuality == 0")
    return cluster_cuts


def thomasROEMask(use_b2bii: bool) -> Dict[str, str]:
    return {
        "name": "ThomasROE",
        "track_cuts": "dr < 0.4 and abs(dz) < 0.4 and nCDCHits > 0"
        if use_b2bii
        else "dr < 0.4 and abs(dz) < 0.4 and nCDCHits > 0",
        "cluster_cuts": " and ".join(thomas_cluster_cuts(use_b2bii)),
    }


roe_mask_list = [
    emptyROEMask,
    cleanROEMask,
    michaelROEMask,
    thomasROEMask,
]  # type: List[Callable]

if __name__ == "__main__":
    b2bii = True
    roe_masks_utils.print_roe_masks(roe_mask_list, use_b2bii=b2bii)
