"""
This file generates useful dictionaries for the low level cut study.

Patrick Ecker 2020
"""

import copy

from typing import Union, Dict, List

import rdstar.online_analysis.decay_channels as base_decay_channels_pkg

from rdstar.cut_information_objects.cut_info import CutInfo
from rdstar.online_analysis.final_state_particles import fsp_cut_sets
from rdstar.online_analysis.final_state_particles import fsp_collection

from rdstar.studies.low_level_cut_study.utilities.final_state_particles_cut_study import (
    fsp_collection as fsp_collection_loose,
    fsp_cut_sets as fsp_cut_sets_loose,
)

from rdstar.studies.low_level_cut_study.utilities.decay_channels_cut_study import (
    D_decay_label_to_cut_info_list as D_loose_dict,
    Dstar_decay_label_to_cut_info_list as D_star_loose_dict,
)


fsp_ntuple_labels_dict: Dict[str, str] = {
    "e_ntuple": "e",
    "mu_ntuple": "mu",
    "piCharged_ntuple": "pi+",
    "piNeutral_from_list_ntuple": "pi0",
    "gamma_ntuple": "gamma",
    "K_ntuple": "K+",
    "K_S0_ntuple": "K_S0",
}

fsp_cut_dict: Dict[str, Dict[str, Union[str, List[CutInfo]]]] = {}
for n_tuple in fsp_ntuple_labels_dict.keys():
    fsps = [copy.copy(fsp) for fsp in fsp_collection.all_fsps if fsp.label == fsp_ntuple_labels_dict[n_tuple]]
    assert len(fsps) == 1
    fsp = fsps[0]
    cut_infos = copy.copy(fsp_cut_sets[fsp.label])

    fsps_loose = [copy.copy(fsp) for fsp in fsp_collection_loose.all_fsps if fsp.label == fsp_ntuple_labels_dict[n_tuple]]
    assert len(fsps_loose) == 1
    fsp = fsps_loose[0]
    cut_infos_loose = copy.copy(fsp_cut_sets_loose[fsp.label])

    fsp_cut_dict.update(
        {
            n_tuple: {
                "cuts": cut_infos,
                "cuts_loose": cut_infos_loose,
                "channel": "",
            }
        }
    )

n_tuple_d_dict: Dict[str, Dict[str, Union[str, Dict[str, List[CutInfo]]]]] = {
    "dzero_ntuple": {
        "cuts": copy.copy(base_decay_channels_pkg.D_decay_label_to_cut_info_list),
        "cuts_loose": copy.copy(D_loose_dict),
        "label": "D0",
    },
    "dcharged_ntuple": {
        "cuts": copy.copy(base_decay_channels_pkg.D_decay_label_to_cut_info_list),
        "cuts_loose": copy.copy(D_loose_dict),
        "label": "D+",
    },
    "dstzero_ntuple": {
        "cuts": copy.copy(base_decay_channels_pkg.Dstar_decay_label_to_cut_info_list),
        "cuts_loose": copy.copy(D_star_loose_dict),
        "label": "D*0",
    },
    "dstcharged_ntuple": {
        "cuts": copy.copy(base_decay_channels_pkg.Dstar_decay_label_to_cut_info_list),
        "cuts_loose": copy.copy(D_star_loose_dict),
        "label": "D*+",
    },
}


D_cut_dict: Dict[str, Dict[str, Union[str, List[CutInfo]]]] = {}
for n_tuple, cut_lists in n_tuple_d_dict.items():
    __cuts: Union[str, Dict[str, List[CutInfo]]] = cut_lists["cuts"]
    assert isinstance(__cuts, dict), __cuts
    _cuts: Dict[str, List[CutInfo]] = __cuts

    __loose_cuts: Union[str, Dict[str, List[CutInfo]]] = cut_lists["cuts_loose"]
    assert isinstance(__loose_cuts, dict), __loose_cuts
    _loose_cuts: Dict[str, List[CutInfo]] = __loose_cuts

    assert isinstance(cut_lists["label"], str), cut_lists["label"]
    __cut_label: str = cut_lists["label"]

    assert _cuts.keys() == _loose_cuts.keys()
    for channel in _cuts.keys():
        if channel.startswith(__cut_label):
            D_cut_dict.update(
                {
                    f"{n_tuple}{channel}": {
                        "cuts": _cuts[channel],
                        "cuts_loose": _loose_cuts[channel],
                        "channel": channel,
                    }
                }
            )

combined_decay_mode_id_dict: Dict[str, int] = {}
combined_decay_mode_id_dict.update(copy.copy(base_decay_channels_pkg.D_decay_label_to_decay_id))
combined_decay_mode_id_dict.update(copy.copy(base_decay_channels_pkg.Dstar_decay_label_to_decay_id))

combined_channel_dict: Dict[str, str] = {}
combined_channel_dict.update(copy.copy(base_decay_channels_pkg.D_decay_channels))
combined_channel_dict.update(copy.copy(base_decay_channels_pkg.Dstar_decay_channels))
combined_channel_dict.update({"": ""})

if __name__ == "__main__":
    for channel, item in base_decay_channels_pkg.Dstar_decay_label_to_cut_info_list.items():
        for c in item:
            print(c)

    for n_tuple_name, info in D_cut_dict.items():
        print(n_tuple_name)
        for c_item in info["cuts"]:
            print("cuts")
            print(c_item)
        for lc_item in info["cuts_loose"]:
            print("loose cuts")
            print(lc_item)

    print(fsp_cut_dict)
    print(D_cut_dict)
    print(combined_decay_mode_id_dict)
