import os

from PyPDF2 import PdfFileMerger
from typing import Union, List, AnyStr

__all__ = [
    "PathType",
    "merge_plot_pdf_files",
]

PathType = Union[str, AnyStr, os.PathLike]


def merge_plot_pdf_files(
    input_paths: List[PathType],
    output_path: PathType,
    overwrite: bool = False,
) -> None:
    pdf_merger = PdfFileMerger()
    assert all(os.path.exists(path) for path in input_paths), [p for p in input_paths if not os.path.exists(p)]
    assert all(os.path.splitext(path)[1] == ".pdf" for path in input_paths), [
        p for p in input_paths if not os.path.splitext(p)[1] == ".pdf"
    ]
    assert os.path.splitext(output_path)[1] == ".pdf", output_path
    if not overwrite:
        assert not os.path.exists(output_path), output_path

    for pdf_path in input_paths:
        pdf_merger.append(pdf_path)

    pdf_merger.write(fileobj=output_path)
    pdf_merger.close()
